/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.amap.lidar.amapvox.visualization.views;

import fr.amap.commons.util.debug.DebugLevel;
import fr.amap.commons.util.debug.Debugger;
import fr.amap.commons.util.swing.FrameView;
import fr.amap.lidar.amapvox.nephos3d.entities.util.VoxelFilter;
import fr.amap.lidar.amapvox.visualization.views.voxelfilter.JXVoxelFilter;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;
import javax.swing.Action;
import javax.swing.BoxLayout;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import org.jdesktop.swingx.JXButton;
import org.jdesktop.swingx.JXCollapsiblePane;
import org.jdesktop.swingx.JXComboBox;
import org.jdesktop.swingx.JXLabel;
import org.jdesktop.swingx.JXMultiThumbSlider;
import org.jdesktop.swingx.JXPanel;
import org.jdesktop.swingx.JXTitledPanel;
import org.jdesktop.swingx.color.GradientThumbRenderer;
import org.jdesktop.swingx.color.GradientTrackRenderer;
import org.jdesktop.swingx.multislider.ThumbListener;
import fr.amap.lidar.amapvox.visualization.controllers.PropertiesController;
import fr.amap.lidar.amapvox.visualization.views.voxelfilter.JXVoxelFilterListener;
import java.util.function.Consumer;

/**
 * Properties view of the project.
 */
public class PropertiesView extends FrameView {

    private PropertiesController controller;

    private JXCollapsiblePane collapsiblePane;
    private JXTitledPanel titledPanel;
    private JPanel scrollPaneContent;
    private JScrollPane scrollPane;

    private JXPanel currentPropertyPanel;
    private JXLabel currentPropertyLabel;
    private JXComboBox currentPropertyComboBox;
    private Consumer<String> onChangePropertyCallback;

    private JXPanel minAndMaxPanel;
    private JXMultiThumbSlider minAndMaxSlider;

    private JXPanel filtersPanel;
    private JXLabel filtersLabel;
    private JXButton filtersButton;
    private final List<JXVoxelFilter> filters;

    /**
     * Creates new form PropertiesFrame
     * @throws java.lang.Exception
     */
    public PropertiesView() throws Exception {
        super();
        this.controller = new PropertiesController();
        this.filters = new ArrayList();
    }
    
    public final void setSelectedProperty(final String property) {
        currentPropertyComboBox.setSelectedItem(property);
    }

    public final void updatePropertyNames() {
        if (controller.getScene() != null) {
            final String[] names = controller.getScene().
                    getVoxelSpaceComponent().getFile().getNames();
            final ComboBoxModel model = new DefaultComboBoxModel(names);
            currentPropertyComboBox.setModel(model);
            currentPropertyComboBox.setSelectedIndex(0);
            
            for (JXVoxelFilter filter : filters) {
                filter.updatePropertyNames(names);
            }
            
            pack();
        }
    }

    public final void setOnChangePropertyCallback(
            final Consumer<String> onChangePropertyCallback) {
        this.onChangePropertyCallback = onChangePropertyCallback;
    }

    public final void clearFilters() throws Exception {
        controller.clearFilters();
        
        for (JXVoxelFilter filter : filters) {
            scrollPaneContent.remove(filter.getContentPanel());
        }
        
        filters.clear();
        collapsiblePane.setCollapsed(true);
        collapsiblePane.setCollapsed(false);
    }

    public final PropertiesController getController() {
        return controller;
    }

    @Override
    protected void initContainer() throws Exception {
        setLocationRelativeTo(getParent());
        setLayout(new BorderLayout());
    }

    @Override
    protected void initComponents() throws Exception {
        /* Init the components */
        initCollapsiblePanel();
        initTitledPanel();
        initScrollPane();
        initCurrentPropertyPanel();
        initFiltersPanel();
//        initMinAndMaxPanel();

        /* Add components to containers */
        collapsiblePane.getContentPane().add(titledPanel, BorderLayout.CENTER);
        titledPanel.getContentContainer().add(scrollPane, BorderLayout.CENTER);
        scrollPaneContent.add(currentPropertyPanel);
        scrollPaneContent.add(filtersPanel);

        /* Reform the frame */
        pack();
    }

    private void initCollapsiblePanel() {
        /* The the collapsible pane */
        collapsiblePane = new JXCollapsiblePane(JXCollapsiblePane.Direction.LEFT);
        collapsiblePane.setAnimated(false);
        collapsiblePane.getContentPane().setLayout(new BorderLayout());

        /* Add the collapsing button */
        final Action toggleAction = collapsiblePane.getActionMap().
                get(JXCollapsiblePane.TOGGLE_ACTION);

        toggleAction.putValue(JXCollapsiblePane.COLLAPSE_ICON, loadIcon("/icons/left.png"));
        toggleAction.putValue(JXCollapsiblePane.EXPAND_ICON, loadIcon("/icons/right.png"));

        final JXButton toggle = new JXButton(toggleAction);
        toggle.setText(null);
        toggle.setPreferredSize(new Dimension(20, 20));

        /* Add the collapsible panel and its button */
        add(collapsiblePane, BorderLayout.CENTER);
        add(toggle, BorderLayout.EAST);
    }

    private void initTitledPanel() {
        titledPanel = new JXTitledPanel();
        titledPanel.getContentContainer().setLayout(new BorderLayout());

        addLocaleCallback((locale) -> {
            titledPanel.setTitle(bundle.getString("propertiesView_Title"));
        });
    }

    private void initScrollPane() {
        scrollPane = new JScrollPane();
        scrollPane.setHorizontalScrollBarPolicy(
                ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollPane.setVerticalScrollBarPolicy(
                ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        scrollPaneContent = new JPanel();
        scrollPaneContent.setLayout(
                new BoxLayout(scrollPaneContent, BoxLayout.PAGE_AXIS));
        scrollPane.setViewportView(scrollPaneContent);
    }

    private void initCurrentPropertyPanel() {
        currentPropertyPanel = new JXPanel(new FlowLayout(FlowLayout.LEFT));
        currentPropertyPanel.setMaximumSize(new Dimension(10000, 30));

        currentPropertyLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            currentPropertyLabel.setText(bundle.getString(
                    "propertiesView_currentPropertyLabel") + " :");
        });
        currentPropertyPanel.add(currentPropertyLabel);

        currentPropertyComboBox = new JXComboBox();
        addLocaleCallback((locale) -> {
            currentPropertyComboBox.setToolTipText(bundle.getString("propertiesView_currentPropertyComboBox_tooltip"));
        });
        currentPropertyComboBox.addActionListener((java.awt.event.ActionEvent evt) -> {
            controller.changeCurrentProperty((String) currentPropertyComboBox.getSelectedItem());
            
            if (onChangePropertyCallback != null) {
                onChangePropertyCallback.accept((String) currentPropertyComboBox.getSelectedItem());
            }
        });

        currentPropertyPanel.add(currentPropertyComboBox);
    }

    private void initMinAndMaxPanel() {
        minAndMaxPanel = new JXPanel(new FlowLayout(FlowLayout.LEFT));

        minAndMaxSlider = new JXMultiThumbSlider();
        final int minThumb = minAndMaxSlider.getModel().addThumb(0f, Color.RED);
        final int maxThumb = minAndMaxSlider.getModel().addThumb(1f, Color.GREEN);
        minAndMaxSlider.setThumbRenderer(new GradientThumbRenderer());
        minAndMaxSlider.setTrackRenderer(new GradientTrackRenderer());
        Dimension dim = new Dimension(200, 16);
        minAndMaxSlider.setPreferredSize(dim);
        minAndMaxSlider.setSize(dim);
        minAndMaxSlider.addMultiThumbListener(new ThumbListener() {
            float lastMin = 0;
            float lastMax = 1;

            @Override
            public void thumbMoved(int thumb, float pos) {
                if (thumb == minThumb) {
                    controller.changeMinAndMax(pos, lastMax);
                    lastMin = pos;
                }
                if (thumb == maxThumb) {
                    controller.changeMinAndMax(lastMin, pos);
                    lastMax = pos;
                }
            }

            @Override
            public void thumbSelected(int thumb) {
            }

            @Override
            public void mousePressed(MouseEvent evt) {
            }
        });

        minAndMaxPanel.add(minAndMaxSlider);
    }

    private void initFiltersPanel() {
        filtersPanel = new JXPanel(new FlowLayout(FlowLayout.LEFT));
        filtersPanel.setMaximumSize(new Dimension(10000, 50));

        filtersLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            filtersLabel.setText(bundle.getString("propertiesView_filtersLabel"));
        });
        filtersPanel.add(filtersLabel);

        filtersButton = new JXButton();
        addLocaleCallback((locale) -> {
            filtersButton.setText(bundle.getString("propertiesView_filtersButton"));
            filtersButton.setToolTipText(bundle.getString("propertiesView_filtersButton_tooltip"));
        });
        filtersButton.addActionListener((e) -> {
            try {
                final JXVoxelFilter filter = new JXVoxelFilter();
                filter.setLocale(getLocale());
                filter.addListener(new JXVoxelFilterListener() {
                    @Override
                    public void onChange(VoxelFilter f) {
                        controller.changeFilter(f);
                        filter.setVoxelFile(controller.
                                getScene().getVoxelSpaceComponent().getFile());
                    }

                    @Override
                    public void onClose(VoxelFilter f) {
                        scrollPaneContent.remove(filter.getContentPanel());
                        filters.remove(filter);
                        
                        if (f != null) {
                            controller.removeFilter(f.getIndex());
                        }
                        
                        collapsiblePane.setCollapsed(true);
                        collapsiblePane.setCollapsed(false);
                    }
                });
                final String[] names = controller.getScene().
                        getVoxelSpaceComponent().getFile().getNames();
                filter.updatePropertyNames(names);
                filter.setDefaultProperty((String) currentPropertyComboBox.getSelectedItem());
                scrollPaneContent.add(filter.getContentPanel());
                collapsiblePane.setCollapsed(true);
                collapsiblePane.setCollapsed(false);
                controller.changeFilter(filter.getFilter());
                filters.add(filter);
                filter.setVoxelFile(controller.getScene().
                    getVoxelSpaceComponent().getFile());
            } catch (Exception ex) {
                Debugger.debugInterface(ex, DebugLevel.ERROR, parentView);
            }
        });
        addLocaleCallback((locale) -> {
            for (JXVoxelFilter filter : filters) {
                filter.setLocale(locale);
            }
        });

        filtersPanel.add(filtersButton);
    }

}
